package com.github.jfcloud.gene.sample.service.biz;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.jfcloud.admin.api.sys.dto.message.MessageUserDTO;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.util.UserUtil;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.SampleEditService;
import com.github.jfcloud.gene.sample.vo.SampleSubmitVo;
import com.github.jfcloud.gene.sys.service.DBSystemPropertiesService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class CommonSampleEditServiceImpl implements SampleEditService {

    private final SampleInfoMapper sampleInfoMapper;
    private final FlowAuditService flowAuditService;
    private final DBSystemPropertiesService systemPropertiesService;
    private final NotifyService notifyService;
    private final UserIdNameCache userIdNameCache;

    @Override
    public void removeBySampleId(Long sampleId) {

    }

    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {

    }

    @Override
    public void generate(Long sampleId) {

    }

    @Override
    public void afterAudit(Long sampleId) {
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        if (sampleInfo == null) {
            return;
        }

        //查询流程前一个状态
        List<FlowAudit> flowAudits = flowAuditService.auditList(sampleId, "sample." + sampleInfo.getType(), false);
        FlowAudit previousAudit = null;
        String previousStatus = "";
        String previousUser = "";
        if (CollUtil.isNotEmpty(flowAudits)) {
            previousAudit = flowAudits.get(0);
            GeneStatusEnum statusEnum = GeneStatusEnum.getByStatus(previousAudit.getFlowStatus());
            previousStatus = statusEnum != null ? statusEnum.getDescription() : "";
            previousUser = userIdNameCache.getNicknameByUserId(previousAudit.getCreateBy());
        }

        String projectType = sampleInfo.projectType();
        String now = DateUtil.format(new Date(), DatePattern.CHINESE_DATE_TIME_PATTERN);

        Long userId;
        String dingMsg;
        GeneStatusEnum statusEnum = Objects.requireNonNull(GeneStatusEnum.getByStatus(sampleInfo.getStatus()));
        switch (statusEnum) {
            case REJECTED:
                //驳回通知申请人
                userId = Long.valueOf(sampleInfo.getCreateBy());
                dingMsg = String.format("【%s】【%s/%s】%s不同意，请修正后再提交", projectType, previousStatus, previousUser, now);
                break;
            case DEPART_LEADER:
                userId = sampleInfo.getDepartLeaderId();
                dingMsg = String.format("【%s】【%s】%s提交，请进行审查", projectType, userIdNameCache.getNicknameByUserId(sampleInfo.getCreateBy()), now);
                break;
            case PROJECT_MANAGEMENT:
                userId = sampleInfo.getProjectManageId();
                dingMsg = String.format("【%s】【%s/%s】%s同意，请进行审查", projectType, previousStatus, previousUser, now);
                break;
            case COMPLETED:
                //执行
                userId = Long.valueOf(sampleInfo.getCreateBy());
                dingMsg = String.format("【%s】【%s/%s】%s同意，请执行项目", projectType, previousStatus, previousUser, now);
                break;
            default:
                log.error("流程项目 ({}) 状态异常 {}", sampleInfo.getProjectName(), statusEnum.getDescription());
                return;
        }

        if (sampleInfo.getVersion() > 1) {
            dingMsg += "（第" + sampleInfo.getVersion() + "次申请）";
        }

        //查询是否需要钉钉通知
        String key = String.format("sample.%s.dingding.enable", sampleInfo.getStatus());
        if (systemPropertiesService.getBooleanValue(key)) {
            MessageUserDTO messageUserDTO = new MessageUserDTO();
            messageUserDTO.setUserId(userId);
            notifyService.sendDingding("样本送检", dingMsg, Collections.singletonList(messageUserDTO));
        }
    }

    /**
     * 项目执行完成通知，添加申请人和检测人员
     *
     * @param sampleId 样本送检Id
     * @param userIds  通知用户Id
     */
    public void afterExecuteNotify(Long sampleId, Set<Long> userIds) {
        //钉钉消息通知申请人、检测人员
        if (!systemPropertiesService.getBooleanValue("sample.afterExecute.dingding.enable")) {
            return;
        }

        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        String now = DateUtil.format(new Date(), DatePattern.CHINESE_DATE_TIME_PATTERN);
        String dingMsg = String.format("【%s】【%s】%s执行项目", sampleInfo.projectType(), UserUtil.getNickName(), now);

        //申请人
        userIds.add(sampleInfo.getApplicantId());
        //检测人员
        if (StrUtil.isNotBlank(sampleInfo.getInspector())) {
            JSON.parseArray(sampleInfo.getInspector()).forEach(item -> {
                JSONObject obj = (JSONObject) item;
                if (obj.containsKey("id")) {
                    userIds.add(obj.getLong("id"));
                }
            });
        }

        List<MessageUserDTO> users = userIds.stream()
                .map(x -> {
                    MessageUserDTO messageUserDTO = new MessageUserDTO();
                    messageUserDTO.setUserId(x);
                    return messageUserDTO;
                })
                .collect(Collectors.toList());
        notifyService.sendDingding("样本送检", dingMsg, users);
    }
}
